<?php

namespace WebsiteBundle\Command;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Query;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Repository\ExperimentRepository;

class UserInactiveCommand extends ContainerAwareCommand
{

  /**
   * @var EntityManager
   */
  private $entityManager;

  protected function configure()
  {
    $this
      ->setName('user:inactive')
      ->setDescription('Places users on inactive');
  }
  
  /**
   * @return EntityManager
   */
  private function getEntityManager() {
    if (empty($this->entityManager)) {
      $this->entityManager = $this->getContainer()->get('doctrine.orm.entity_manager');
    }

    return $this->entityManager;
  }
  
  /**
   * @param string $entityClass
   * @return EntityRepository
   */
  private function getRepository($entityClass) {
    if (empty($this->entityManager)) {
      $this->getEntityManager();
    }

    return $this->entityManager->getRepository($entityClass);
  }

  /**
   * @param OutputInterface $output
   * @param $message
   * @param string $type
   */
  private function printOutput(OutputInterface $output, $message, $type = 'comment') {
    $output->writeln('<'.$type.'>'.date('Y-m-d H:i:s').' - '.$message.'</'.$type.'>');
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $this->printOutput($output, 'Starting execution');
    //Get all ongoing experiments
    $experimentRepo = $this->getRepository(Experiment::class);
    $experiments = $experimentRepo->getCurrentExperiments(Query::HYDRATE_OBJECT);

    //Place the participants on inactive
    $participantRepo = $this->getRepository(Participant::class);

    foreach($experiments as $experiment) {
      $this->printOutput($output, 'Processing experiment \''.$experiment->getName().'\'');

      $participantRepo->setParticipantsToInactive($experiment);

      $counts = $experimentRepo->getActiveParticipantCount($experiment);

      foreach($counts as $count) {
        if ($count['active']) {
          $experiment->setActiveParticipantCount($count['participantCount']);
          break;
        } else {
          $experiment->setActiveParticipantCount(0);
        }
      }

      $this->entityManager->persist($experiment);
      $this->entityManager->flush();
    }

    $this->printOutput($output, 'Done.', 'info');
  }

}
