<?php

namespace WebsiteBundle\Entity;

use Bazookas\AdminBundle\Entity\CMSUser;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use WebsiteBundle\Security\Roles;

/**
 * @ORM\Entity(repositoryClass="WebsiteBundle\Repository\ResearcherRepository")
 * @ORM\HasLifecycleCallbacks
 */
class Researcher extends CMSUser {

  /**
   * @var ArrayCollection
   * @ORM\ManyToMany(targetEntity="Experiment", mappedBy="researchers")
   */
  private $experiments;

  /**
   * @ORM\Column(type="string")
   * @var string
   */
  private $phoneNumber;

  public function __construct() {
    parent::__construct();

    $this->experiments = new ArrayCollection();
    $this->locale = 'nl';
  }

  /**
   * @return ArrayCollection
   */
  public function getExperiments()
  {
    return $this->experiments;
  }

  /**
   * @param Experiment[] $experiments
   * @return Researcher
   */
  public function setExperiments($experiments)
  {
    if (is_array($experiments)) {
      $experiments = new ArrayCollection($experiments);
    }

    $this->experiments = $experiments;

    return $this;
  }

  /**
   * @param Experiment $experiment
   * @return $this
   */
  public function addExperiment(Experiment $experiment) {
    $experiment->addResearcher($this);
    $this->experiments->add($experiment);
    return $this;
  }

  /**
   * @param Experiment $experiment
   * @return $this
   */
  public function removeExperiment(Experiment $experiment) {
    $experiment->removeResearcher($this);
    $this->experiments->removeElement($experiment);
    return $this;
  }

  /**
   * @return string
   */
  public function getPhoneNumber()
  {
    return $this->phoneNumber;
  }

  /**
   * @param string $phoneNumber
   * @return Researcher
   */
  public function setPhoneNumber($phoneNumber)
  {
    $this->phoneNumber = $phoneNumber;

    return $this;
  }

}
