<?php

namespace WebsiteBundle\Form\Admin;

use Doctrine\ORM\EntityRepository;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\ExperimentCondition;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\Test;

class ParticipantAdminType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $experiments = $options['experiments'];

    $builder
      ->add('active', CheckboxType::class, [
        'label' => 'admin.entities.participant.fields.active',
        'required' => false,
      ])
      ->add('name', TextType::class, [
        'label' => 'admin.entities.participant.fields.name',
      ])
      ->add('surname', TextType::class, [
        'label' => 'admin.entities.participant.fields.surname',
      ])
      ->add('phoneNumber', TextType::class, [
        'label' => 'admin.entities.participant.fields.phoneNumber',
        'required' => false,
      ])
      ->add('email', TextType::class, [
        'label' => 'admin.entities.participant.fields.email',
      ]);

    $entity = $builder->getData();
    $isEdit = $entity->getId() != null;
    $isExperimentEditable = !$isEdit || ($isEdit && $entity->getSessions()->isEmpty());

    if ($isExperimentEditable) {
      $builder
        ->add('experiment', EntityType::class, [
          'label' => 'admin.entities.participant.fields.experiment',
          'class' => Experiment::class,
          'choice_label' => 'name',
          'choices' => $experiments,
          'disabled' => count($builder->getData()->getSessions()) > 1,
          'attr' => [
            'class' => 'js-experiment select2'
          ]
        ]);
    }

    if ($isEdit && count($entity->getSessions()) < 3) {
      $builder
        ->add('condition', EntityType::class, [
          'label' => 'admin.entities.participant.fields.condition',
          'class' => ExperimentCondition::class,
          'choice_label' => function (ExperimentCondition $condition) {
            return $condition->getTestGroup() . ' - ' . $condition->getGamificationLevel();
          },
          'choice_attr' => function ($val, $key, $index) {
            return [
              'data-conditionally-visible' => json_encode([
                '.js-experiment' => '$element.val() == "' . $val->getExperiment()->getId() . '"'
              ]),
              'data-conditionally-detached' => true,
            ];
          },
          'query_builder' => function (EntityRepository $er) use ($isExperimentEditable, $entity) {
            if ($isExperimentEditable) {
              return $er->createQueryBuilder('conditions');
            } else {
              $experimentId = $entity->getExperiment()->getId();
              $qb = $er->createQueryBuilder('conditions');
              $qb
                ->andWhere($qb->expr()->eq('conditions.experiment', ':experiment'))
                ->setParameter('experiment', $experimentId)
              ;
              return $qb;
            }
          },
        ]);
    }

    $builder
      ->add('plainPassword', RepeatedType::class, [
        'label' => 'admin.entities.participant.fields.plainPassword',
        'type' => PasswordType::class,
        'invalid_message' => 'password.passwordsDontMatch',
        'options' => ['attr' => ['class' => 'password-field']],
        'required' => is_null($builder->getData()->getId()),
        'first_options' => ['label' => 'admin.entities.participant.fields.plainPassword'],
        'second_options' => ['label' => 'admin.entities.participant.fields.plainPasswordRepeat'],
      ]);
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => Participant::class,
      'experiments' => [],
      'validation_groups' => ['addOrEdit'],
      'translation_domain' => 'admin',
    ]);
  }
}
