<?php

namespace WebsiteBundle\Form\Admin;

use Doctrine\ORM\EntityRepository;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\ExperimentCondition;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\Test;

class ParticipantBulkEditAdminType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('experiment', EntityType::class, [
        'label' => 'admin.entities.participant.fields.experiment',
        'class' => Experiment::class,
        'choice_label' => 'name',
        'choices' => $options['experiments'],
        'attr' => [
          'class' => 'js-experiment select2'
        ]
      ])
      ->add('condition', EntityType::class, [
        'label' => 'admin.entities.participant.fields.condition',
        'class' => ExperimentCondition::class,
        'choice_label' => function ($condition) {
          return $condition->getTestGroup() . ' - ' . $condition->getGamificationLevel();
        },
        'choice_attr' => function ($val, $key, $index) {
          return [
            'data-conditionally-visible' => json_encode([
              '.js-experiment' => '$element.val() != "' . $val->getExperiment()->getId() . '"'
            ]),
            'data-conditionally-detached' => true,
          ];
        },
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => Participant::class,
      'experiments' => [],
      'translation_domain' => 'admin'
    ]);
  }
}
