<?php

namespace WebsiteBundle\Form\Admin;

use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Bazookas\CommonBundle\Form\Type\LocaleType;
use Doctrine\ORM\EntityManager;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Researcher;

class ResearcherAdminType extends AbstractType
{

  private $entityManager;
  private $auth;

  public function __construct(EntityManager $entityManager, AuthorizationChecker $auth) {
    $this->entityManager = $entityManager;
    $this->auth = $auth;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('enabled', CheckboxType::class, array(
        'label' => 'admin.entities.researcher.fields.enabled',
        'required' => false
      ))

      ->add('name', TextType::class, array(
        'label' => 'admin.entities.researcher.fields.name'
      ))
      ->add('surname', TextType::class, array(
        'label' => 'admin.entities.researcher.fields.surname'
      ))
      ->add('email', TextType::class, array(
        'label' => 'admin.entities.researcher.fields.email'
      ))
      ->add('phoneNumber', TextType::class, [
        'label' => 'admin.entities.researcher.fields.phoneNumber'
      ])

      ->add('username', TextType::class, array(
        'label' => 'admin.entities.researcher.fields.username'
      ))
    ;

    if ($builder->getData()->getId() == null) {
      $builder
        ->add('plainPassword', RepeatedType::class, array(
          'label' => 'admin.entities.cMSUser.fields.plainPassword',
          'type' => PasswordType::class,
          'invalid_message' => 'password.passwordsDontMatch',
          'options' => array('attr' => array('class' => 'password-field')),
          'required' => true,
          'first_options'  => array('label' => 'admin.entities.CMSUser.fields.plainPassword'),
          'second_options' => array('label' => 'admin.entities.CMSUser.fields.plainPasswordRepeat')
        ))
      ;
    }

    if ($this->auth->isGranted('ROLE_ADMIN_RESEARCHER')) {
      $builder
        ->add('groups', EntityType::class, array(
          'label' => false,
          'class' => CMSUserGroup::class,
          'choice_label' => 'name',
          'required' => true,
          'multiple' => true,
          'expanded' => true,
          'choices' => $this->getGroups()
        ))
        ->add('experiments', EntityType::class, array(
          'label' => false,
          'class' => Experiment::class,
          'choice_label' => 'name',
          'multiple' => true,
          'by_reference' => false,
          'required' => false
        ))
      ;
    }
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => Researcher::class,
      'validation_groups' => array('Default'),
      'grouping' => [
        'admin.entities.cMSUser.forms.panelHeader1' => [
          'enabled',
          'name',
          'surname',
          'email',
          'phoneNumber',
          'username'
        ]
      ],
      'translation_domain' => 'admin'
    ));
  }

  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    if (isset($options['grouping'])) {
      if ($form->has('plainPassword')) {
        $options['grouping']['admin.entities.cMSUser.forms.panelHeader1'][] = 'plainPassword';
      }

      if ($form->has('groups')) {
        $options['grouping']['admin.entities.cMSUser.forms.panelHeader2'] = [
          'groups'
        ];
        $options['grouping']['admin.entities.researcher.forms.panelHeader3'] = [
          'experiments'
        ];
      }

      $view->vars['grouping'] = $options['grouping'];
    }
  }

  private function getGroups() {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('g')
      ->from('Bazookas\AdminBundle\Entity\CMSUserGroup', 'g')
    ;
    $results = $qb->getQuery()->getResult();

    foreach($results as $key => $result) {
      //if a group contains the role SUPER_ADMIN, then the user must also be super admin before being able to
      //choose it as an option in the form
      if (!$this->auth->isGranted('ROLE_SUPER_ADMIN') && $result->hasRole('ROLE_SUPER_ADMIN')) {
        unset($results[$key]);
      }
    }
    return $results;
  }
}
