<?php

namespace WebsiteBundle\Services\Data;

use Bazookas\APIFrameworkBundle\Exception\ApiException;
use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\PsychEducationAnswer;
use WebsiteBundle\Entity\PsychEducationPanel;
use WebsiteBundle\Exception\ExceptionMap;

class PsychEducationDataService extends BaseDataService
{
  /**
   * @var EntityManager
   */
  private $entityManager;

  public function __construct(EntityManager $entityManager) {
    $this->entityManager = $entityManager;
  }
  
  public function submitPsychEducationQuestions(ParameterBag $params, ParameterBag $extraParams) {
    $participant = $extraParams->get('user');

    if ($participant->hasCompletedPsychEducation()) {
      throw new ApiException('User has already completed the psych education', ExceptionMap::PSYCH_EDU_COMPLETED, Response::HTTP_BAD_REQUEST);
    }

    $experiment = $participant->getExperiment();

    if (empty($experiment)) {
      throw new AccessDeniedException();
    }

    $panels = $this->getPsychEducationPanels($experiment);

    $data = $params->get('data');
    foreach($data as $answerData) {
      $panelId = $answerData['panelId'];
      $panel = null;
      foreach($panels as $panel) {
        if ($panel->getId() == $panelId) {
          break;
        }
      }

      if (!empty($panel) && $panel->getType() != PsychEducationPanel::TYPE_TEXT) {
        $selectedResponseIds = $answerData['responseIds'];
        foreach($selectedResponseIds as $selectedResponse) {
          $answer = new PsychEducationAnswer();
          $answer->setParticipant($participant);

          foreach($panel->getResponses() as $response) {
            if ($response->getId() == $selectedResponse) {
              $answer->setResponse($response);
              break;
            }
          }
          $this->entityManager->persist($answer);
        }
      }
    }

    $participant->setCompletedPsychEducation(true);
    $this->entityManager->persist($participant);
    $this->entityManager->flush();
    
    //Need to return an empty response
    return [];
  }

  private function getPsychEducationPanels(Experiment $experiment) {
    $entityRepo = $this->entityManager->getRepository(PsychEducationPanel::class);
    return $entityRepo->getPsychEducationPanels($experiment, Query::HYDRATE_OBJECT);
  }
}
