<?php
namespace Bazookas\AdminBundle\DataFixtures\ORM;

use Bazookas\CommonBundle\Security\Roles;
use Doctrine\Common\DataFixtures\AbstractFixture;
use Doctrine\Common\DataFixtures\FixtureInterface;
use Doctrine\Common\Persistence\ObjectManager;
use Symfony\Component\Console\Output\ConsoleOutput;
use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Entity\CMSUserGroup;

class AdminFixtures extends AbstractFixture implements FixtureInterface
{
  public function load(ObjectManager $manager)
  {
    $superGroup = new CMSUserGroup();
    $superGroup->setName('Super Administrator');
    $superGroup->setRoles([Roles::ROLE_SUPER_ADMIN]);
    $manager->persist($superGroup);

    $adminGroup = new CMSUserGroup();
    $adminGroup->setName('Administrator');
    $adminGroup->setRoles([Roles::ROLE_ADMIN]);
    $manager->persist($adminGroup);

    $user = new CMSUser();
    $password = base64_encode(random_bytes(6));

    $user
      ->setUsername('admin')
      ->setPlainPassword($password)
      ->setName('admin')
      ->setSurname('Bazookas')
      ->setLocale('nl')
      ->addGroup($superGroup)
      ->setEmail('play@bazookas.be')
      ->setEnabled(true)
    ;
    $this->setReference('user-super-admin', $user);

    $manager->persist($user);
    $manager->flush();

    $output = new ConsoleOutput();
    $output->writeln([
      "<comment>  > User 'admin' created with password '" . $password . "'</comment>",
    ]);
  }

  public function getOrder()
  {
    return 50;
  }
}
