<?php

namespace Bazookas\AdminBundle\DependencyInjection;

use Bazookas\AdminBundle\EventListener\Entity\AuditLogListener;
use Bazookas\AdminBundle\EventListener\Entity\VersioningListener;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\HttpKernel\DependencyInjection\Extension;
use Symfony\Component\DependencyInjection\Loader;

/**
 * This is the class that loads and manages your bundle configuration.
 *
 * @link http://symfony.com/doc/current/cookbook/bundles/extension.html
 */
class BazookasAdminExtension extends Extension
{
  /**
   * {@inheritdoc}
   */
  public function load(array $configs, ContainerBuilder $container)
  {
    $configuration = new Configuration();
    $config = $this->processConfiguration($configuration, $configs);


    $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
    $loader->load('services.yml');

    $menu = !empty($config) ? $config['menu'] : array();
    $container->setParameter('bazookas_admin.menu', $menu);

    if (isset($config['audit']) && !empty($config['audit'])) {
      $this->addAuditServices($container, $config['audit']);
    }

    if (isset($config['audit']) && isset($config['audit']['enableVersioning']) && $config['audit']['enableVersioning']) {
      $this->addVersioningServices($container);
    }
  }

  private function addAuditServices($container, $auditConfig) {
    if (isset($auditConfig['logging']) && !empty($auditConfig['logging'])) {
      $this->addLoggingService($container, $auditConfig['logging']);
    }
  }

  private function addLoggingService($container, $logConfig) {
    //Add the logging class as a parameter to the container
    $container->setParameter('bazookas_admin.audit.logging.logClass', $logConfig['logClass']);

    //Register the audit log service
    $container
      ->register('BazookasAdminBundle.EventListener.AuditLogListener', AuditLogListener::class)
      ->addArgument('%bazookas_admin.audit.logging.logClass%')
      ->addArgument(new Reference('security.token_storage'))
      ->addTag('doctrine.event_listener', ['event' => 'onFlush'])
      ->addTag('doctrine.event_listener', ['event' => 'prePersist'])
    ;
  }

  private function addVersioningServices($container) {
    //register the versioning service
    $container
      ->register('BazookasAdminBundle.EventListener.VersioningListener', VersioningListener::class)
      ->addArgument(new Reference('security.token_storage'))
      ->addTag('doctrine.event_listener', ['event' => 'onFlush'])
      ->addTag('doctrine.event_listener', ['event' => 'prePersist'])
    ;
  }

}
