<?php

namespace Bazookas\AdminBundle\Entity\Traits;

use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Entity\Interfaces\Versionable;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

trait VersionableTrait
{
  /**
   * @ORM\Column(type="string")
   */
  protected $entityId;

  /**
   * @var ArrayCollection
   */
  protected $children;

  /**
   * @var Versionable|null
   */
  protected $parent;

  /**
   * @ORM\ManyToOne(targetEntity="Bazookas\AdminBundle\Entity\CMSUser")
   * @var CMSUser
   */
  protected $creator;

  /**
   * @return string
   */
  public function getEntityId()
  {
    return $this->entityId;
  }

  /**
   * @param string $entityId
   * @return $this
   */
  public function setEntityId($entityId)
  {
    $this->entityId = $entityId;

    return $this;
  }

  /**
   * @return Versionable|null
   */
  public function getParent() {
    return $this->parent;
  }

  /**
   * @param Versionable $parent
   * @return $this
   */
  public function setParent(Versionable $parent = null) {
    $this->parent = $parent;
    return $this;
  }

  /**
   * @param Versionable[]|ArrayCollection $children
   * @return $this
   */
  public function setChildren($children) {
    if (is_array($children)) {
      $children = new ArrayCollection($children);
    }

    $this->children = $children;
    return $this;
  }

  /**
   * @return ArrayCollection
   */
  public function getChildren() {
    return $this->children;
  }

  /**
   * @param Versionable $child
   * @return $this
   */
  public function addChild(Versionable $child) {
    $this->children->add($child);
    return $this;
  }

  /**
   * @param BaseCMSUser $creator
   * @return $this;
   */
  public function setCreator(BaseCMSUser $creator) {
    $this->creator = $creator;
    return $this;
  }

}
