<?php

namespace Bazookas\AdminBundle\Form;

use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;

use Doctrine\ORM\EntityManager;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Bazookas\CommonBundle\Form\Type\LocaleType;

/**
 * @deprecated
 *
 * Class EditCMSUserAdminType
 * @package Bazookas\AdminBundle\Form
 */
class EditCMSUserAdminType extends AbstractType
{
  private $entityManager;
  private $auth;

  public function __construct(EntityManager $entityManager, AuthorizationChecker $auth) {
    $this->entityManager = $entityManager;
    $this->auth = $auth;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    //TODO see:
    //http://symfony.com/doc/current/bundles/FOSUserBundle/overriding_forms.html

      $builder
        ->add('enabled', CheckboxType::class, array(
          'label' => 'admin.entities.CMSUser.fields.enabled',
          'required' => false
        ))

        ->add('name', TextType::class, array(
          'label' => 'admin.entities.CMSUser.fields.name'
        ))
        ->add('surname', TextType::class, array(
          'label' => 'admin.entities.CMSUser.fields.surname'
        ))
        ->add('email', TextType::class, array(
          'label' => 'admin.entities.CMSUser.fields.email'
        ))
        ->add('username', TextType::class, array(
          'label' => 'admin.entities.CMSUser.fields.username'
        ))

        //TODO This is hardcoded! make it not so!
        ->add('locale', LocaleType::class, array(
          'label' => 'admin.entities.CMSUser.fields.locale'
        ))
        ->add('groups', EntityType::class, array(
          'label' => 'admin.entities.CMSUser.fields.groups',
          'class' => 'BazookasAdminBundle:CMSUserGroup',
          'choice_label' => 'name',
          'required' => true,
          'multiple' => true,
          'expanded' => true,
          'choices' => $this->getGroups()
        ))
      ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => 'Bazookas\AdminBundle\Entity\CMSUser',
      'translation_domain' => 'admin'
    ));
  }

  private function getGroups() {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('g')
      ->from('Bazookas\AdminBundle\Entity\CMSUserGroup', 'g')
    ;
    $results = $qb->getQuery()->getResult();

    foreach($results as $key => $result) {
      //if a group contains the role SUPER_ADMIN, then the user must also be super admin before being able to
      //choose it as an option in the form
      if (!$this->auth->isGranted(Roles::ROLE_SUPER_ADMIN) && $result->hasRole(Roles::ROLE_SUPER_ADMIN)) {
        unset($results[$key]);
      }
    }
    return $results;
  }
}
