## AdminElement elements

- [AlertElement](#alertelement)
- [BreadcrumbElement](#breadcrumbelement)
- [CalendarElement](#calendarelement)
- [ErrorElement](#errorelement)
- [FormElement](#formelement)
- [GraphElement](#graphelement)
- [IconTextElement](#icontextelement)
- [ImageElement](#imageelement)
- [ProfileElement](#profileelement)
- [ProgressedStatisticElement](#progressedstatisticelement)
- [StatisticElement](#statisticelement)
- [StatisticOverviewElement](#statisticoverviewelement)
- [StatisticPanelElement](#statisticPanelElement)
- [TextElement](#textelement)
- [UserStatisticElement](#userstatisticelement)
- [ListGroupElement](#listgroupelement)

#### AlertElement

An alert element is an element which adds a bootstrap alert to the page.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:alertElement.html.twig',  //the template to be used by the widget
      'class' => '',  //a class on the parent element of the widget
      'message' => '',  //The message to be shown
      'type' => self::TYPE_INFO,  //The type of message
      'isDismissible' => true //Is the alert dismissible?
    ]);
  }
```

Available types:
```php
  const TYPE_INFO = 'info';
  const TYPE_DANGER = 'danger';
  const TYPE_SUCCESS = 'success';
  const TYPE_PRIMARY = 'primary';
```

Example:
```php
  $element = new AlertElement([], [
    'message' => 'admin.errors.miniGameNotFound',
    'type' => AlertElement::TYPE_DANGER,
  ]);
```

#### BreadcrumbElement

An element to be added in a list of breadcrumbs. Should be used in association with `BreadcrumbContainer` (see [Containers](./containers.md#breadcrumbcontainer))

Example:

```php
// create an array to store breadcrumbs
$breadcrumbs = [];

// add a breadcrumb to the array
$breadcrumbs[] = new BreadcrumbElement([], [
  'route' => $route,
  'routeParams' => [
    '_action' => BaseAdminListController::ACTION_LIST,
    'filters' => [],
  ],
  'label' => 'admin.media.breadcrumb.root'
]);

// create a container to render all breadcrumbs in a nice list
$container = new BreadcrumbContainer($breadcrumbs);
```

#### CalendarElement

The calendar element renders a basic 12 month calendar (current month + 11). It is currently not possible to change the range of the months to be rendered. 

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:calendarElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'primary',
      'icon' => 'fa-user',
      'iconClass' => 'fa fa-4x mar-all',
      'data' => []
    ]);
  }
``` 

The calendar expects the data to be in the format:
```php
[
    'date' => new \DateTime(),
    'label' => '',
    'colorClass' => 'success',
    'link' => ''
]
```

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:errorElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'danger',
      'icon' => 'fa fa-exclamation-triangle',
      'iconClass' => 'fa-4x mar-all'
    ]);
  }
```

Example:
```php

$data = [
    [
        'date' => new \DateTime(),
        'label' => 'an event',
        'colorClass' => 'success',
        'link' => ''
    ]
]

$calendar = new CalendarElement([], ['data' => $data]);
```

#### ErrorElement

This element is used to render an error widget.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:errorElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'danger',
      'icon' => 'fa fa-exclamation-triangle',
      'iconClass' => 'fa-4x mar-all'
    ]);
  }
```

Example:
```php
$element = new ErrorElement([], [
  'label' => 'dashboard.main.tip.noTipError',
]);
```

#### FormElement
This element is used to render a form in a panel.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:formElement.html.twig',
      'class' => '',
      'form' => null
    ]);
  }
```

Example:
```php
$element = new FormElement([], [
  'form' => $mySymfonyForm,
]);
```

#### GraphElement

An element to render a graph item.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard/Elements:graphElement.html.twig',
      'class' => 'canvas-responsive js-graph',
      'type' => self::TYPE_BAR,
      'options' => [],
      'data' => [],
      'icon' => 'fa-pie',
      'iconClass' => 'fa'
    ]);
  }
```

available graph types:

```php
  const TYPE_RADAR = 'radar';
  const TYPE_BAR = 'bar';
  const TYPE_HORIZONTAL_BAR = 'horizontalBar';
  const TYPE_PIE = 'pie';
  const TYPE_DOUGHNUT = 'doughnut';
```

Example:

```php
$graphTitle = $this->translator->trans('admin.entities.appUser.views.stats.miniGameGraphTitle');

$dataDefinition = GraphElement::generateDefaultDataDefinition(GraphElement::TYPE_RADAR);
$dataDefinition['datasets'][0]['data'] = $data;
$dataDefinition['datasets'][0]['label'] = $graphTitle;
$dataDefinition['labels'] = $labels;

return new RowContainer(
  new GraphElement([], [
    'type' => GraphElement::TYPE_PIE,
    'data' => $dataDefinition,
    'options' => [
      'title' => [
        'display' => true,
        'text' => $graphTitle
      ],
      'tooltips' => [
        'callbacks' => [
          'label' => 'percentageTooltip'
        ]
      ],
      'animation' => [
       'onComplete' => 'addPercentageToSegment',
       'onProgress' => 'addPercentageToSegment',
     ],
     'hover' => [
       'animationDuration' => 0,
     ],
    ]
  ])
}
```

A graph item can also be given the following callback functions:

`percentageTooltip` => add a `%` sign to the end of the value in the tooltip
`addPercentageToSegment` => add the pie charts percent value to the drawn segment.
`addValueToBar` => add the bar graphs value to the drawn segment.

#### IconTextElement

Renders an icon element.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:iconTextElement.html.twig',
      'class' => 'text-lg text-semibold text-main',
      'iconClass' => 'text-lg text-semibold text-main'
    ]);
  }
```

A sample implementation:
```php
$element = new IconTextElement([], [
  'value' => 'the text'
]);
```

#### ImageElement

Renders an image.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'roles' => ['ROLE_ADMIN'],
      'template' => 'BazookasAdminBundle:Dashboard\Elements:imageElement.html.twig',
      'class' => 'img-md',
    ]);
  }
```

```php
new ImageElement([], [
  'avatarId' => $user->getAvatarId(),
  'class' => 'img-md img-circle mar-btm',
  'template' => 'WebsiteBundle:Dashboard\\AppUser:imageElement.html.twig',
]);
```

#### ProfileElement

Renders a profile element.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:profileElement.html.twig',
      'class' => 'panel',
    ]);
  }
```

Example:
```php
new ProfileElement(
  [
    new ListContainer(
      [
        new TextElement([], [
          'value' => $user->getStoryMode(),
          'label' => 'admin.entities.appUser.fields.storyMode',
          'colClass' => 'col-xs-4 pad-all',
          'template' => 'WebsiteBundle:Dashboard\\AppUser:storyModeElement.html.twig',
        ]),
        new TextElement([], [
          'value' => $user->getCurrentTier() + 1,
          'label' => 'admin.entities.appUser.fields.currentTier',
          'colClass' => 'col-xs-4 pad-all',
        ]),
        new TextElement([], [
          'value' => $user->getRankScore(),
          'label' => 'admin.entities.appUser.fields.rankScore',
          'colClass' => 'col-xs-4 pad-all',
        ]),
      ]
    ),
    new ListContainer(
      [
        new TextElement([], [
          'value' => $user->getRank(),
          'label' => 'admin.entities.appUser.fields.rank',
          'colClass' => 'col-xs-4 pad-all',
        ]),
        new TextElement([], [
          'value' => $user->getGamesPlayed(),
          'label' => 'admin.entities.appUser.fields.gamesPlayed',
          'colClass' => 'col-xs-4 pad-all',
        ]),
      ]
    ),
  ],
  [
    'image' => new ImageElement([], [
      'avatarId' => $user->getAvatarId(),
      'class' => 'img-md img-circle mar-btm',
      'template' => 'WebsiteBundle:Dashboard\\AppUser:imageElement.html.twig',
    ]),
    'name' => $user->getUsername(),
    'role' => null,
  ]
);
```

#### ProgressedStatisticElement
Used to render a statistic widget with a progress bar beneath it.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:progressedStatisticElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'primary',
      'icon' => 'fa-user',
      'iconClass' => 'fa fa-4x',
      'progressValue' => 0,
      'progressLabel' => '',
    ]);
  }
```

Example:
```php
$element = new ProgressedStatisticElement([], [
    'value' => $count,
    'label' => 'dashboard.main.login.activeUsers',
    'icon' => 'fa-user',
    'progressValue' => $percent,
    'progressLabel' => 'dashboard.main.login.progress',
    'panelColor' => 'success'
]);
```

#### StatisticElement
Used to render a statistic widget (For use with a single statistic).

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:statisticElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'primary',
      'icon' => 'fa-user',
      'iconClass' => 'fa fa-4x mar-all',
    ]);
  }
```

Example:
```php
$element = new StatisticElement([], [
  'value' => $teamCount,
  'label' => 'dashboard.main.team.count',
  'icon' => 'fa-users',
  'panelColor' => 'primary'
]);

```

#### StatisticOverviewElement
Used to render a widget containing a series of statistics (rendered as a table within a panel). The element will automatically highlight the largest value in green. The lowest value is highlighted in red. All other values are highlighted in blue.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:statisticOverviewElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'primary',
      'icon' => 'fa-user',
      'iconClass' => 'fa fa-4x mar-all',
      'data' => [],
      'highlightValues' => true
    ]);
  }
```

Example:
```php
$data = [
    [
        'label' => 'a label',
        'value' => 3
    ]
];

$element = new StatisticOverviewElement([], [
  'label' => 'dashboard.main.playedCount.label',
  'value' => $total,
  'panelColor' => 'info',
  'icon' => 'fa-gamepad',
  'data' => $data
]);
```

#### StatisticPanelElement


//TODO not implemented


#### TextElement

Renders a text element.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:textElement.html.twig',
      'class' => '',
    ]);
  }
```

Example:
```php
new TextElement([], [
  'value' => $user->getStoryMode(),
  'label' => 'admin.entities.appUser.fields.storyMode',
  'colClass' => 'col-xs-4 pad-all',
  'template' => 'WebsiteBundle:Dashboard\\AppUser:storyModeElement.html.twig',
]);
```

#### UserStatisticElement
Used to render user related statistics.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard\Elements:userStatisticElement.html.twig',
      'class' => 'panel panel-colorful',
      'panelColor' => 'success'
    ]);
  }
```

Example:
```php
$element = new UserStatisticElement([], [
  'value' => 1000,
  'label' => 'Highscore'
]);
```

#### ListGroupElement
Used to render a nifty list group. The content of the page is set through the `content` option. This option contains an array of elements such that the key is the translation key for the header element and the value is the key for the text element. `header => text`

```php
new ListGroupElement([], [
  'navName' => 'admin.entities.copy.help.navName',
  'content' => [
    'admin.entities.copy.help.content.item1.header' => 'admin.entities.copy.help.content.item1.text'
  ]
])
```
