## Export Service

This service is included in this bundle and is used to export data from the database to a csv-file.

When using this service in your controller, you need to set the export-config in your controller-function and then call the exportAction provided by the ListBaseController.

This can be done in the following way:

```php
public function exportAction(Request $request) {

  $this->exportConfig = [
    'entity' => $this->config->entity,
    'fields' => [
      'id' => 'Id',
      // you can pass false when you don't want to export the field,
      // but just make sure all joins are in place 
      'timeline.app' => false,
      'options.answers.user.profile' => 'admin.field.name',
      'options.answers.user.profile.firstName' => 'admin.field.name',
      'options.answers.user.profile.lastName' => 'admin.field.surname',
      'options.answers.user.profile.email' => 'admin.field.email',
      'options.answers.user.profile.gender' => 'admin.field.gender',
      'options.answers.user.profile.profileUrl' => 'admin.field.profileUrl',
      'options.answers.user.profile.profilePicture' => 'admin.field.picture',
    ],
    'fileName' => 'poll_users_export.csv',
    'groupBy' => 'e_options_answer.id',
    'conditions' => [
      'e.id = :id',
    ],
    'parameters' => [
      'id' => $this->id,
    ],
  ];
  return parent::exportAction($request);
}

```
When configuring the exportConfig, there are 3 elements that are required, and 2 that are optional.

Required elements:
- `entity`: this contains the base entity where you want to select your data from. In this example, we are using the entity related to the controller where we are calling the export-function.
- `fields`: this should be an array with all the fields of data that you want to export from the database. The key of the array must be a field in the database, while the value is the text that will be displayed in the header of the csv-file. In this example, we are using the translation files for the headers of the csv-file.
- `fileName`: this is the name that will be given to the generated csv-file.

Optional elements:
- `conditions`: if there are no conditions provided, the service will fetch all the data from the requested fields. If the data needs to pass certain requirements, these requirements should be provided in this element. In the example above, we are exporting user-data for all users who voted on a certain poll. This is done with a simple condition where the id of a poll element must match the provided id. More on conditions below.
- `parameters`: to prevent sql-injection, we are using parameters. This element should contain the values that need to be inserted into the parameters that were used in the conditions-element. For some conditions it is not required to use a parameter, for example if the condition is that a certain field cannot be NULL. However, in most cases parameters will be required.

#### Conditions

In the example above, we provided a condition in the following way:

```php
'conditions' => [
  'e.id = :id',
]
```
When there are multiple requirements that need to be fulfilled, each requirement can be an element in the array:
```php
'conditions' => [
  'e.id = :id',
  'e_options_answers_user_profile.gender like :gender',
]
```
As can be seen in the example above, we provided a requirement for the base entity and a requirement for a related entity.
When we have a requirement for a field of the base entity, we refer to it in the following way:
```php
'e.{field} = :{field}'
```
where you replace {field} with the relevant name.

When we have a requirement for a field of a related entity, we refer to it in the following way:
```php
'e_{related}_{related2}.{field} = :{field}
```
where {related} is  an entity related to the base entity, {related2} is an entity related to {related} and {field} is a field belonging to entity {related2}.

In the previous examples, the data had to fulfill all the provided requirements. In some cases, the data must fulfill one OR another requirement.
This can be done in the following way:
```php
'conditions' => [
  new \Doctrine\ORM\Query\Expr\Orx(['e_options_answers_user_profile.gender like :gender', 'e_options_answers_user_profile.email IS NOT NULL']),
]
```
In this example, the service will only export the data where the user is the specified gender or if their email is not null.
