<?php

namespace Bazookas\APIFrameworkBundle\Controller;

use Bazookas\APIFrameworkBundle\Services\Validation\ValidationService;
use Symfony\Component\HttpFoundation\Request;
use Bazookas\APIFrameworkBundle\Controller\Base\BaseController ;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\Validator\Constraints\DateTime;

class AppSettingsController extends BaseController
{
  public function getDataService($dataServiceFunction)
  {
    return $this->get('APIFrameworkBundle.Data.AppSettingsDataService');
  }


  /**
   * Get a list of all app settings, optionally filtered by a timestamp
   *
   * # Example response body
   * *Note:* no primary "id" is returned, because "key" is used as primary key
   *
   *     {
   *      "data": [
   *        {
   *          "key": "test",
   *          "setting": "this is a test setting"
   *        },
   *        {
   *          "key": "test_2",
   *          "setting": "this is a second test settings"
   *        }
   *      ],
   *      "lastModified": "1483967169"
   *     }
   *
   * @ApiDoc(
   *   section="Settings",
   *  resource=true,
   *  description="",
   *  requirements={},
   *  headers={
   *   {
   *     "name"="Authorization",
   *     "description"="Used to send the access token to authenticate the app to the API. The header is formatted as follows: 'Authorization':'Bearer < token >'.",
   *     "required" = true
   *   },
   *  },
   *  parameters={
   *   {
   *    "name"="newer",
   *    "dataType"="timestamp",
   *    "required"=false,
   *    "description"="get settings created or modified after given timestamp"
   *   },
   *  },
   *  statusCodes={
   *    200="Returned when successful",
   *    400="Missing or invalid parameter"
   *  },
   *  tags={
   *    "requires-client_credentials" = "#ffa500",
   *  }
   *)
   */
  public function getAction(Request $request) {
    $retrieveParams = array(
      'newer' => [
        'constraint' => [new DateTime(['format' => 'U'])],
        'convertToType' => ValidationService::TYPE_DATETIME,
        'conversionFormat' => 'U',
      ]
    );

    return $this->handleRequest($request, $retrieveParams, 'getAll', Response::HTTP_OK);
  }
}
