<?php

namespace Bazookas\APIFrameworkBundle\Entity;

use Bazookas\APIFrameworkBundle\Util\APIFileUploadCallback;
use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;

/**
 * APIFileUpload
 *
 * @ORM\Table(name="api_file_upload")
 * @ORM\Entity(repositoryClass="Bazookas\APIFrameworkBundle\Repository\APIFileUploadRepository")
 * @ORM\HasLifeCycleCallbacks
 */
class APIFileUpload extends BaseEntity
{
  /**
   * @var string
   * @ORM\Column(type="string", length=255)
   */
  private $hash;

  /**
   * @var int
   * @ORM\Column(type="string", length=255)
   */
  private $uploadIndex;

  /**
   * @var string
   * @ORM\Column(type="string", length=255, nullable=true)
   */
  private $tempFileName;

  /**
   * @var string
   * @ORM\Column(type="string", length=255)
   */
  private $targetDirectory;

  /**
   * @var string
   * @ORM\Column(type="string", length=255, nullable=true)
   */
  private $preferredFileName;

  /**
   * @var string
   * @ORM\Column(type="string", length=255, nullable=true)
   */
  private $file;

  /**
   * @var bool
   * @ORM\Column(type="boolean")
   */
  private $completed = false;

  /**
   * @var bool
   * @ORM\Column(type="boolean")
   */
  private $overwrite = false;

  /**
   * @var APIFileUploadCallback
   * @ORM\Column(type="object", nullable=true)
   */
  private $onUploadCallback;

  /**
   * @var APIFileUploadCallback
   * @ORM\Column(type="object", nullable=true)
   */
  private $onCompleteCallback;

  /**
   * @var string
   * @ORM\Column(type="string", length=255, nullable=true)
   */
  private $mimeTypes;

  /****************************
   * UNMAPPED FIELDS          *
   * for passing to callbacks *
   ****************************/
  private $finalUrl;
  private $relativeUrl;

  /**
   * @return int
   */
  public function getId()
  {
    return $this->id;
  }

  /**
   * @param string $hash
   * @return APIFileUpload
   */
  public function setHash($hash)
  {
    $this->hash = $hash;

    return $this;
  }

  /**
   * @return string
   */
  public function getHash()
  {
    return $this->hash;
  }

  /**
   * @param integer $uploadIndex
   * @return APIFileUpload
   */
  public function setUploadIndex($uploadIndex)
  {
    $this->uploadIndex = $uploadIndex;

    return $this;
  }

  /**
   * @return int
   */
  public function getUploadIndex()
  {
    return $this->uploadIndex;
  }

  /**
   * @param string $tempFileName
   * @return APIFileUpload
   */
  public function setTempFileName($tempFileName)
  {
    $this->tempFileName = $tempFileName;

    return $this;
  }

  /**
   * @return string
   */
  public function getTempFileName()
  {
    return $this->tempFileName;
  }

  /**
   * @param string $targetDirectory
   * @return APIFileUpload
   */
  public function setTargetDirectory($targetDirectory)
  {
    $this->targetDirectory = $targetDirectory;
    return $this;
  }

  /**
   * @return string
   */
  public function getTargetDirectory()
  {
    return $this->targetDirectory;
  }

  /**
   * @return string
   */
  public function getPreferredFileName($fallback = null)
  {
    if (!empty($this->preferredFileName)) {
      return $this->preferredFileName;
    }
    return $fallback;
  }

  /**
   * @param string $preferredFileName
   * @return APIFileUpload
   */
  public function setPreferredFileName($preferredFileName)
  {
    $this->preferredFileName = $preferredFileName;
    return $this;
  }

  /**
   * @return string
   */
  public function getFile()
  {
    return $this->file;
  }

  /**
   * @param string $file
   * @return APIFileUpload
   */
  public function setFile($file)
  {
    $this->file = $file;
    return $this;
  }

  /**
   * @param boolean $completed
   * @return APIFileUpload
   */
  public function setCompleted($completed)
  {
    $this->completed = $completed;

    return $this;
  }

  /**
   * Get completed
   * @return bool
   */
  public function isCompleted()
  {
    return $this->completed;
  }

  /**
   * @return boolean
   */
  public function isOverwrite()
  {
    return $this->overwrite;
  }

  /**
   * @param boolean $overwrite
   * @return APIFileUpload
   */
  public function setOverwrite($overwrite)
  {
    $this->overwrite = $overwrite;
    return $this;
  }



  /**
   * @param string $onUploadCallback
   * @return APIFileUpload
   */
  public function setOnUploadCallback($onUploadCallback)
  {
    $this->onUploadCallback = $onUploadCallback;

    return $this;
  }

  /**
   * Get onUploadCallback
   * @return string
   */
  public function getOnUploadCallback()
  {
    return $this->onUploadCallback;
  }

  /**
   * @param string $onCompleteCallback
   * @return APIFileUpload
   */
  public function setOnCompleteCallback($onCompleteCallback)
  {
    $this->onCompleteCallback = $onCompleteCallback;

    return $this;
  }

  /**
   * @return string
   */
  public function getOnCompleteCallback()
  {
    return $this->onCompleteCallback;
  }

  /**
   * @return mixed
   */
  public function getFinalUrl()
  {
    return $this->finalUrl;
  }

  /**
   * @param mixed $finalUrl
   * @return APIFileUpload
   */
  public function setFinalUrl($finalUrl)
  {
    $this->finalUrl = $finalUrl;
    return $this;
  }

  /**
   * @return mixed
   */
  public function getRelativeUrl()
  {
    return $this->relativeUrl;
  }

  /**
   * @param mixed $relativeUrl
   * @return APIFileUpload
   */
  public function setRelativeUrl($relativeUrl)
  {
    $this->relativeUrl = $relativeUrl;
    return $this;
  }

  public function getMimeTypes()
  {
    if (empty($this->mimeTypes)) {
      return '/.+/'; // allow everything
    }
    return $this->mimeTypes;
  }

  public function setMimeTypes($mimeTypes)
  {
    $this->mimeTypes = $mimeTypes;
    return $this;
  }


}
