<?php
namespace Bazookas\APIFrameworkBundle\Exception\Base;

use Bazookas\CommonBundle\Exception\Base\BaseException;
use Exception;
use Bazookas\APIFrameworkBundle\Exception\Interfaces\ApiExceptionInterface;
use Bazookas\CommonBundle\Services\Exception\Interfaces\LoggableExceptionInterface;
use Symfony\Component\HttpFoundation\Response;

abstract class BaseApiException extends BaseException implements ApiExceptionInterface, LoggableExceptionInterface
{
  protected $loggable = false;
  protected $statusCode;

  // Redefine the exception so message isn't optional
  public function __construct($message, $name = ApiExceptionMap::GENERAL_ERROR, $statusCode = Response::HTTP_INTERNAL_SERVER_ERROR, Exception $previous = null) {
    $this->statusCode = $statusCode;

    // make sure everything is assigned properly
    parent::__construct($message, $name, $previous);
  }

  public function setIsLoggable($loggable) {
    $this->loggable = $loggable;
    return $this;
  }

  public function isLoggable() {
    return $this->loggable;
  }

  public function getHttpStatusCode() {
    return $this->statusCode;
  }

  public function getName() {
    $refClass = new \ReflectionClass($this);
    return $refClass->getShortName();
  }

  public abstract function getLoggingSeverity();

}
