# AppCopy

Similar to AppSettings, you can configure the API to provide 

## CMS

Add the following route (optionally prefixed):

```
bazookas_api_framework_app_copy_admin:
    path:     /app-copy/{_action}/{_id}
    defaults:
      _controller: BazookasAPIFrameworkBundle:Admin\AppCopyAdmin:index
      _action: ~
      _id: ~
```

The rest will be handled by the api framework.

## Configuring your project

1. Create entities which extend `Bazookas\APIFrameworkBundle\Entity\Base\BaseAppCopy` and `Detail`

```
<?php
namespace APIBundle\Entity;

use Bazookas\APIFrameworkBundle\Entity\Base\BaseAppCopy;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class AppCopy extends BaseAppCopy
{

  /**
   * @var ArrayCollection<AppCopyDetail>
   * @ORM\OneToMany(targetEntity="AppCopyDetail", mappedBy="detailParent")
   */
  protected $details;

}
```

```
<?php
namespace APIBundle\Entity;

use Bazookas\APIFrameworkBundle\Entity\Base\BaseAppCopyDetail;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class AppCopyDetail extends BaseAppCopyDetail
{

  /**
   * @var AppCopy $detailParent
   * @ORM\ManyToOne(targetEntity="AppCopy", inversedBy="details")
   * @ORM\JoinColumn(name="key_string", referencedColumnName="key_string")
   */
  protected $detailParent;

}

```

2. Add the following route:

```
api_v1_get_copy:
  path: /{_locale}/translations
  methods: [GET]
  requirements:
    _locale: '%bazookas_api_framework.app_copy.allowed_locales%'
  defaults:
    _controller: BazookasAPIFrameworkBundle:AppCopy:get
```

3. Let the framework know which entity to use (in config.yml):

```
bazookas_api_framework:
  app_copy:
    entity: APIBundle\Entity\AppCopy
    # required when working with the admin bundle
    details_entity: APIBundle\Entity\AppCopyDetail
    # optionally provide API specific language constraint (defaults to %requiredlocales%).
    # allowed_locales: nl|fr
```
