<?php

namespace Bazookas\APIFrameworkBundle\Services\Data;

use Bazookas\APIFrameworkBundle\Formatter\AppSettingsFormatter;
use Bazookas\APIFrameworkBundle\Formatter\Base\BaseFormatter;
use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Bazookas\CommonBundle\Repository\Interfaces\RepositoryInterface;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\HttpFoundation\ParameterBag;

// NOTE: this service is defined in BazookasAPIFrameworkExtension.php
class AppSettingsDataService extends BaseDataService
{
  public function __construct(EntityManagerInterface $entityManager, $entityClass) {
    $this->repository = $entityManager->getRepository($entityClass);

    // don't call the parent constructor, because we can't be sure the repository extends the correct repository interface
    //parent::__construct($repository, null);
  }
  
  public function getAll(ParameterBag $params) { // no $extraParams for now
    $newer = $params->get('newer');

    /**
     * @var QueryBuilder $qb
     */
    $qb = $this->repository->createQueryBuilder('s');
    if ($params->has('newer')) {
      $qb
        ->andWhere($qb->expr()->orX(
          $qb->expr()->gt('s.created', ':newer'),
          $qb->expr()->gt('s.modified', ':newer')
        ))
        ->setParameter('newer', $newer);
    }
    $result = $qb->getQuery()->getArrayResult();

    $formatter = new AppSettingsFormatter();
    $result = $formatter->format($result);

    $lastModified = $this->getLastModified($newer, $result, true);
    if (!empty($lastModified)) {
      $lastModified = $lastModified->format('U');
    }

    // return the resulting settings
    return [
      'data' => $result,
      'lastModified' => $lastModified,
    ];
  }
}
