<?php

namespace Bazookas\CommonBundle\Entity\Base;

use Bazookas\CommonBundle\Entity\Interfaces\AccessControlInterface;
use Bazookas\CommonBundle\Entity\Interfaces\EntityInterface;
use Bazookas\CommonBundle\Security\Roles;
use Bazookas\CommonBundle\Entity\Interfaces\Timestampable;
use Doctrine\ORM\Mapping as ORM;
use FOS\UserBundle\Model\User as FOSBaseUser;

use Bazookas\CommonBundle\Entity\Traits\BaseEntityTrait;

/**
* @ORM\MappedSuperclass
*/
abstract class BaseCMSUser extends FOSBaseUser implements EntityInterface, AccessControlInterface, Timestampable  {

  use BaseEntityTrait;

  const ROLE_DEFAULT = Roles::ROLE_CMS_USER;

  /**
  * @ORM\Column(type="integer")
  * @ORM\Id
  * @ORM\GeneratedValue(strategy="AUTO")
  */
  protected $id;

  /**
   * @ORM\Column(type="string", nullable=true)
   **/
  protected $name;
  /**
   * @ORM\Column(type="string", nullable=true)
   **/
  protected $surname;

  public function __construct() {
    parent::__construct();
  }

  public function getName() {
    return $this->name;
  }

  public function setName($name) {
    $this->name = $name;
    return $this;
  }

  public function getSurname() {
    return $this->surname;
  }

  public function setSurname($name) {
    $this->surname = $name;
    return $this;
  }

  /**
   * Return name and surname
   **/
  public function getFullName() {
    $fullName = "$this->name $this->surname";
    return $fullName !== ' ' ? $fullName : null;
  }

  /**
   * Get the url of the avatar, if no avatar is uploaded, default to gravatar
   **/
  public function getAvatarURL() {
    // always default to gravatar since uploads are not yet implemented
    return 'https://www.gravatar.com/avatar/'.
      md5(strtolower(trim($this->email))).
      '?d=retro'; // use the retro gravatar style when no image is found
  }

  public function getRoleRequiredForView() {
    return Roles::ROLE_ADD_USER;
  }

}
