<?php
namespace Bazookas\CommonBundle\Exception\Base;

use Exception;
use Bazookas\CommonBundle\Services\Exception\Interfaces\LoggableExceptionInterface;
use Symfony\Component\HttpFoundation\Response;

abstract class BaseConnectionException extends \RuntimeException implements LoggableExceptionInterface
{
  protected $message;
  protected $loggable = false;
  protected $statusCode;

  // Redefine the exception so message isn't optional
  public function __construct($message, $code = 0, $statusCode = Response::HTTP_INTERNAL_SERVER_ERROR, Exception $previous = null) {
    $this->message = $message;
    $this->statusCode = $statusCode;
    // make sure everything is assigned properly
    parent::__construct($message, $code, $previous);
  }

  // custom string representation of object
  public function __toString() {
    return get_class($this) . ": {$this->message}\n in '".$this->getFile()."' at line ".$this->getLine().".";
  }

  public abstract function getName();

  public function setIsLoggable($loggable) {
    $this->loggable = $loggable;
    return $this;
  }

  public function isLoggable() {
    return $this->loggable;
  }

  public abstract function getLoggingSeverity();

  public function getHttpStatusCode()
  {
    return Response::HTTP_INTERNAL_SERVER_ERROR;
  }
}
