<?php

namespace Bazookas\CommonBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;

class LocaleType extends AbstractType
{
  protected $requiredLocales;

  public function __construct($requiredLocales) {
    $this->requiredLocales = $requiredLocales;
  }

  public function configureOptions(OptionsResolver $resolver) {
    $resolver->setDefaults(array(
      'label' => 'common.entity.field.locale',
      'choices' => $this->requiredLocalesAsChoiceArray($this->requiredLocales),
      'attr' => [
        'class' => ' '
      ],
      'translation_domain' => 'admin'
    ));
  }

  /**
 * {@inheritdoc}
 */
  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    if (count($view->vars['choices']) < 2) {
      $class = isset($view->vars['attr']['class']) ? $view->vars['attr']['class'] : '';
      $view->vars['attr']['class'] = $class . ' hide';

      $labelClass = isset($view->vars['label_attr']['class']) ? $view->vars['label_attr']['class'] : '';
      $view->vars['label_attr']['class'] = $labelClass . ' hide';
    }
  }

  public function getParent()
  {
    return ChoiceType::class;
  }

  public function getName()
  {
    return 'locale_type';
  }

  private function requiredLocalesAsChoiceArray($requiredLocales) {
    $requiredLocales = explode('|', $requiredLocales);

    $locales = [];
    foreach($requiredLocales as $locale) {
      $locales[$locale] = $locale;
    }

    return $locales;
  }
}
