<?php
namespace Bazookas\CommonBundle\Twig;

use Bazookas\CommonBundle\Utility\Utility;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\PersistentCollection;

class CommonExtensions extends \Twig_Extension {

  private $translator;

  public function __construct($translator) {
    $this->translator = $translator;
  }

  public function getFilters()
  {
    return array(
      new \Twig_SimpleFilter('deepmerge', 'array_replace_recursive'),
      new \Twig_SimpleFilter('recursivemerge', 'array_merge_recursive'),
      new \Twig_SimpleFilter('md5', 'md5'),
      new \Twig_SimpleFilter('getByLocale', array($this, 'getByLocale')),
      new \Twig_SimpleFilter('timeToReadable', array($this, 'timeToReadable')),
    );
  }

  public function getByLocale($list, $locale) {
    if ($list instanceof ArrayCollection || $list instanceof PersistentCollection) {
      $list = $list->toArray();
    }

    if (is_array($list) && !empty($list)) {
      foreach($list as $listItem) {
        if (property_exists($listItem, 'locale')) {
          $itemLocale = $listItem->getLocale();
          if (is_object($itemLocale) && $itemLocale->getLocale() === $locale) {
            return $listItem;
          } else {
            if ($itemLocale === $locale) {
              return $listItem;
            }
          }
        }
      }

      return $list[0];
    }

    return null;
  }

  public function timeToReadable($value) {
    list($days, $hours, $minutes, $seconds) = Utility::convertTimeToReadable($value);

    return $this->translator->trans('common.util.convertTimeToReadable', [
      '%day%' => $days,
      '%hour%' => $hours,
      '%min%' => $minutes,
      '%sec%' => $seconds
    ]);
  }

  public function getFunctions()
  {
    return array(
      new \Twig_SimpleFunction('get_translation_catalogue', array($this->translator->getCatalogue(), 'all')),
    );
  }

  public function getName() {
    return 'bazookas_common_deep_merge';
  }
}
