<?php
namespace Bazookas\IntegrationsBundle\Exception\Base;

use Bazookas\IntegrationsBundle\Exception\IntegrationsExceptionMap;
use Exception;
use Bazookas\IntegrationsBundle\Exception\Interfaces\IntegrationExceptionInterface;
use Bazookas\CommonBundle\Services\Exception\Interfaces\LoggableExceptionInterface;
use Symfony\Component\HttpFoundation\Response;

abstract class BaseIntegrationException extends \RuntimeException implements IntegrationExceptionInterface, LoggableExceptionInterface
{
  protected $message;
  private $loggable = false;
  protected $name;

  // Redefine the exception so message isn't optional
  public function __construct($message, $name = IntegrationsExceptionMap::INTEGRATION_EXCEPTION, $statusCode = Response::HTTP_INTERNAL_SERVER_ERROR, Exception $previous = null) {
    $this->message = $message;
    $this->name = $name;
    $this->statusCode = $statusCode;

    // make sure everything is assigned properly
    parent::__construct($message, 0, $previous);
  }

  // custom string representation of object
  public function __toString() {
    $integration = $this->getIntegration();
    return $integration . ": {$this->message}\n in '".$this->getFile()."' at line ".$this->getLine().".";
  }

  public function setName($name) {
    $this->name = $name;
    return $this;
  }
  public function getName() {
    if (!empty($this->name)) {
      return $this->name;
    }
    $reflectionClass = new \ReflectionClass($this);
    return $reflectionClass->getShortName();
  }

  public function setIsLoggable($loggable) {
    $this->loggable = $loggable;
    return $this;
  }

  public function isLoggable() {
    return $this->loggable;
  }

  public abstract function getLoggingSeverity();

}
