<?php

namespace Bazookas\MediaBundle\Entity\Base;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\Validator\Constraints as Assert;
use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Bazookas\MediaBundle\Entity\Interfaces\MediaEntityInterface;
use Bazookas\CommonBundle\Entity\Traits\LocalisedTrait;
use Bazookas\CommonBundle\Entity\Interfaces\LocalisedEntityInterface;
use Bazookas\MediaBundle\Entity\Image;
use Bazookas\MediaBundle\Entity\Video;

/**
* @ORM\Entity
* @ORM\Table(name="media")
* @ORM\InheritanceType("SINGLE_TABLE")
* @ORM\DiscriminatorColumn(name="type", type="string")
* @ORM\DiscriminatorMap({
*   "TYPE_IMAGE" = "Bazookas\MediaBundle\Entity\Image",
*   "TYPE_VIDEO" = "Bazookas\MediaBundle\Entity\Video"
* })
* @ORM\HasLifecycleCallbacks
*/
abstract class BaseMedia extends BaseEntity implements MediaEntityInterface, LocalisedEntityInterface, \JsonSerializable {

  use LocalisedTrait;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $title;

  /**
  * @ORM\Column(type="text", nullable=true)
  */
  protected $description;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $path;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $relativeUrl;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $url;

  protected $file;

  /**
  * @ORM\ManyToOne(targetEntity="Bazookas\MediaBundle\Entity\MediaFolder", inversedBy="mediaItems")
  */
  protected $mediaFolder;

  public function getTitle() {
    return $this->title;
  }

  public function setTitle($title) {
    $this->title = $title;
    return $this;
  }

  public function getDescription() {
    return $this->description;
  }

  public function setDescription($description) {
    $this->description = $description;
    return $this;
  }

  public function getPath() {
    return $this->path;
  }

  public function setPath($path) {
    $this->path = $path;
    return $this;
  }

  public function getRelativeUrl() {
    return $this->relativeUrl;
  }

  public function setRelativeUrl($url) {
    $this->relativeUrl = $url;
    return $this;
  }

  public function getUrl() {
    return $this->url;
  }

  public function setUrl($url) {
    $this->url = $url;
    return $this;
  }

  public function getType() {
    switch(true) {
      case $this instanceof Image:
        return 'TYPE_IMAGE';
      case $this instanceof Video:
        return 'TYPE_VIDEO';
    }
  }

  public function getFile() {
    return $this->file;
  }

  public function setFile($file) {
    $this->file = $file;

    if ($file instanceof UploadedFile) {
      $this->setModified(new \DateTime('now'));
    }
    return $this;
  }

  public function getMediaFolder() {
    return $this->mediaFolder;
  }

  public function setMediaFolder($folder) {
    $this->mediaFolder = $folder;
    return $this;
  }

  public abstract function getDirectory();

  public function JsonSerialize()
  {
    $vars = get_object_vars($this);
    return $vars;
  }

  public function toHumanReadable($locale = 'nl') {
    return get_class($this)."'".$this->title."'";
  }

  public function getRoleRequiredForView() {
    return 'ROLE_MEDIA_ADMIN';
  }
}
