<?php

namespace Bazookas\MediaBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Bazookas\MediaBundle\Entity\Base\BaseMedia;

/**
* @ORM\Entity
* @ORM\HasLifecycleCallbacks
*/
class Video extends BaseMedia {

  const PROVIDER_VIMEO = 'PROVIDER_VIMEO';
  const PROVIDER_YOUTUBE = 'PROVIDER_YOUTUBE';
  const PROVIDER_NATIVE = 'PROVIDER_NATIVE';

  /**
  * @ORM\Column(type="array", nullable=true)
  */
  protected $resolutions;

  /**
  * @ORM\Column(type="integer", nullable=true)
  */
  protected $duration;

  /**
  * @ORM\Column(type="string")
  */
  protected $provider;

  /**
  * @ORM\Column(type="string")
  */
  protected $providerId;

  /**
  * @Assert\File(maxSize="200M",
  * mimeTypes = {
  *    "video/mp4"
  *  },
  *  maxSizeMessage = "media.error.fileSize",
  *  mimeTypesMessage = "media.error.invalidType"
  * )
  */
  protected $file;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $thumbnail;

  public function getResolutions() {
    return $this->resolutions;
  }

  public function setResolutions($resolutions) {
    $this->resolutions = $resolutions;
    return $this;
  }

  public function getDuration() {
    return $this->duration;
  }

  public function setDuration($duration) {
    $this->duration = $duration;
    return $this;
  }

  public function getProvider() {
    return $this->provider;
  }

  public function setProvider($provider) {
    $this->provider = $provider;
    return $this;
  }

  public function getProviderId() {
    return $this->providerId;
  }

  public function setProviderId($id) {
    $this->providerId = $id;
    return $this;
  }

  public function getThumbnail() {
    return $this->thumbnail;
  }

  public function setThumbnail($thumbnail) {
    $this->thumbnail = $thumbnail;
    return $this;
  }

  public function getDirectory() {
    return 'videos';
  }

  //TODO: use constants for the quality format array
  //But we can't use constants cuz office doesn't support php7
  public static function getResolutionFormats() {
    return array(
      'QUALITY_LOW' => array(  //320p
        'w' => 480,
        'h' => 320,
        'fps' => 30
      ),
      'QUALITY_STANDARD' => array(  //480p
        'w' => 640,
        'h' => 480,
        'fps' => 30
      ),
      'QUALITY_HIGH' => array(  //720p
        'w' => 1280,
        'h' => 720,
        'fps' => 30
      ),
      'QUALITY_VERY_HIGH' => array( //1080p
        'w' => 1920,
        'h' => 1080,
        'fps' => 30
      ),
    );
  }

}
